global with sharing class API_Octopush {
    
    public static Integer PART_THRESHOLD = Integer.valueOf(Label.TECH_SEND_PART_THRESHOLD);
    public static Integer PART_SPLITTED_NB = Integer.valueOf(Label.TECH_SEND_PART_SPLITTED_NB);

    private static globalSmsResult getOctopushResponse(HTTPResponse httpResponse, String httpRequestBody, map<String, String> smsInputs) {
        // Parse request body
        map<String, String> requestParam = new map<String, String>{};
        requestParam.putAll(smsInputs);
        for(String aParam : httpRequestBody.split('&')) {
            list<String> keyVal = aParam.split('=');
            requestParam.put(keyVal[0], keyVal.size() > 1 ? keyVal[1] : '');
        }
        globalSmsResult qsr = new globalSmsResult();
        // Parse XML body response
        Dom.Document doc = httpResponse.getBodyDocument();       
        Dom.XMLNode octopush = doc.getRootElement();
           
        // Isolate error code
        Integer SMSSentOK = 0; 
        String nbOfSentSMS = octopush.getChildElement('number_of_sendings', null) != null ? octopush.getChildElement('number_of_sendings', null).getText(): '0';           
        String totalCost = octopush.getChildElement('cost', null) != null ? octopush.getChildElement('cost', null).getText(): 'null';        
        String billingCurrency = octopush.getChildElement('currency_code', null) != null ? octopush.getChildElement('currency_code', null).getText(): 'null'; 
         
        Dom.Document doc2 = httpResponse.getBodyDocument();
        list<smsResult> srList = new list<smsResult>();
        // Case 1: Set of mobile
        for(dom.XmlNode node : doc2.getRootElement().getChildElements()) {
          if(node.getName()=='failures' || node.getName()=='successs') {
              for(dom.XmlNode nodeSMSResult : node.getChildElements()) {
                 String errorCode = nodeSMSResult.getChildElement('error_code', null) != null ? nodeSMSResult.getChildElement('error_code', null).getText() : null;
                 String recipient = nodeSMSResult.getChildElement('recipient', null) != null ? nodeSMSResult.getChildElement('recipient', null).getText() : null;
                 String country_code = nodeSMSResult.getChildElement('country_code', null) != null ? nodeSMSResult.getChildElement('country_code', null).getText() : null;
                 String cost = nodeSMSResult.getChildElement('cost', null) != null ? nodeSMSResult.getChildElement('cost', null).getText() : null;
                 String sms_needed = nodeSMSResult.getChildElement('sms_needed', null) != null ? nodeSMSResult.getChildElement('sms_needed', null).getText() : null;                     

                 smsResult sr = new smsResult();
                 sr.recipient = recipient;
                 sr.error_code = errorCode;
                 sr.country_code = country_code;
                 sr.cost = cost;
                 sr.sms_needed = sms_needed;
                 sr.status = (errorCode == null) ? 'Transmitted to Octopush' : 'Error';
                 sr.errorReason = getErrorMessageByCode(sr.error_code, 'EN');
                 srList.add(sr);
                 // Count success & failure
                 if(sr.error_code == null)
                     SMSSentOK++;   
             }
           }
        }
        // Case 2: Only 1 error code is retrieved
        if(srList.isempty()) {
             smsResult sr = new smsResult();
             sr.recipient = requestParam.get('sms_recipients');
             sr.error_code = octopush.getChildElement('error_code', null) != null ? octopush.getChildElement('error_code', null).getText(): null;
             sr.country_code = null;
             sr.cost = null;
             sr.sms_needed = null;
             sr.status = (sr.error_code == null) ? 'Transmitted to Octopush' : 'Error';
             sr.errorReason = getErrorMessageByCode(sr.error_code, 'EN');
             srList.add(sr);
        }
        qsr.smsResult = srList;
        qsr.totalSms = srList.size();
        qsr.successSms = SMSSentOK;
        qsr.failureSms = srList.size()-SMSSentOK;
        qsr.httpResBody = httpResponse.getBody()+'';
        qsr.totalCost = totalCost;
        qsr.billingCurrency = billingCurrency; 
        qsr.templateID = requestParam.get('templateID');
        qsr.message = requestParam.get('sms_text');
        qsr.receiverType = requestParam.get('receiverType');
        qsr.sendingDate = requestParam.containsKey('sending_date') ? requestParam.get('sending_date') : DateTime.Now()+'';
        qsr.defferedSending = requestParam.containsKey('sms_mode') && requestParam.get('sms_mode') == '2';
        qsr.senderName = requestParam.get('sms_sender');
        qsr.purpose = requestParam.containsKey('transactional') && requestParam.get('transactional') == '1' ? 'Transactional' : 'Marketing';
        
        if(! requestParam.containsKey('estimation')) {
            storeSMSHistory(new list<globalSmsResult>{qsr});
        }
        return qsr;
    } 
    
    private static void storeSMSHistory(list<globalSmsResult> qsrList) {
        // Get octopush settings
        Octopush_Settings__c os = Utils_Octopush.getOctopushSettings();
        if(os.History_Recording__c && Schema.sObjectType.SMS_History__c.isCreateable()) { // Store only if configured in Octo settings
            list <SMS_History__c> smsHistoryList = new list <SMS_History__c>();
            for(globalSmsResult qsr :qsrList) {
                for(smsResult sr : qsr.smsResult) {
                    //Datetime sendingDate = datetime.valueOf(qsr.sendingDate);
                    SMS_History__c sh = new SMS_History__c(            
                        Mobile_Number__c = sr.recipient,
                        SMS_Template__c = (qsr.templateID == 'null' || String.isBlank(qsr.templateID) ? null : qsr.templateID),
                        SMS_Message__c = qsr.message,
                        Receiver_Type__c = qsr.receiverType,
                        Status__c = sr.status,
                        Error_Reason__c = sr.errorReason,
                        Sending_Date__c =  datetime.valueOfGmt(qsr.sendingDate),
                        Deffered_Sending__c = qsr.defferedSending,
                        Sender_Name__c = qsr.senderName,
                        SMS_Purpose__c = qsr.purpose
                    );
                    smsHistoryList.add(sh);
                }
            }
            insert smsHistoryList;
        }
    }
    
    public static globalSmsResult getEstimationStatus(String action, String userID, list<globalSmsResult> estimationSMSStoringList, String additionnalParam) {         
        // <?xml version="1.0" encoding="UTF-8"?> <octopush> <error_code>000</error_code> <cost>0.448</cost> <number_of_sendings>7</number_of_sendings> <ticket>SMS14298000555b600e9d8ca</ticket> <text>&lt;![CDATA[test]]&gt;</text> </octopush
        globalSmsResult qsr = new globalSmsResult();
        Octopush_Settings__c os = Utils_Octopush.getOctopushSettings();
        
        String body = 'user_login='+os.User_Name__c+'&';
        if(String.isNotBlank(additionnalParam))
            body += additionnalParam;
        body += 'api_key='+os.API_KEY__c+'&';
        body += 'request_mode='+((Boolean) os.Real_SMS_Sending__c ? 'real':'simu')+'&';
        body += 'action='+action+'&';     
        body += 'user_batch_id='+userID;
           
         
        HttpRequest req = new HttpRequest();
        req.setHeader('Content-type','application/x-www-form-urlencoded');
        req.setMethod('POST');
        req.setEndpoint('https://www.octopush-dm.com/api_sf_ss/sms_batch');
        req.setBody(body);
        Http http = new Http();
        HTTPResponse res = http.send(req);
        // Parse XML body response
        Dom.Document doc = res.getBodyDocument();
        Dom.XMLNode octopush = doc.getRootElement();
        qsr.estimate_error_code = octopush.getChildElement('error_code', null) != null ? octopush.getChildElement('error_code', null).getText(): null; 
        qsr.totalCost = octopush.getChildElement('cost', null) != null ? octopush.getChildElement('cost', null).getText(): null; 
        String successSms = octopush.getChildElement('number_of_sendings', null) != null ? octopush.getChildElement('number_of_sendings', null).getText(): null; 
        qsr.successSms = successSms !=null ? Integer.valueOf(successSms) : null;
        qsr.httpResBody = res.getBody();
        system.debug('OCTOPUSH LOG: XML RESPONSE BODY: '+res.getBody());
        
        // Store if action is send
        if(action == 'send') {
            storeSMSHistory(estimationSMSStoringList);
            qsr.successSms = qsr.failureSms = qsr.totalSms = 0;
            for (globalSmsResult tempQsr: estimationSMSStoringList) {
                qsr.successSms += tempQsr.failureSms == 0 ? 1 : 0;
                qsr.failureSms += tempQsr.failureSms == 1 ? 1 : 0;
                qsr.totalSms++; 
            }       
        }
        return qsr;
    }
    
    public static globalSmsResult sendSMS(Boolean estimation, String requestBody, map<String, String> smsInputs){         
         globalSmsResult qsr = new globalSmsResult();
         Octopush_Settings__c os = Utils_Octopush.getOctopushSettings();
         
         requestBody += 'user_login='+os.User_Name__c+'&';
         requestBody += 'api_key='+os.API_KEY__c+'&';
         requestBody += 'sms_type='+'WWW'+'&';
         requestBody += 'request_mode='+((Boolean) os.Real_SMS_Sending__c ? 'real':'simu');      
         
         HttpRequest req = new HttpRequest();
         req.setHeader('Content-type','application/x-www-form-urlencoded');
         req.setMethod('POST'); 
         String octoEndPoint = (estimation) ? 'https://www.octopush-dm.com/api_sf_ss/sms_parts' : 'https://www.octopush-dm.com/api_sf_ss/sms_send';
         req.setEndpoint(octoEndPoint); 
         req.setBody(requestBody);
         Http http = new Http();
         system.debug('OCTOPUSH LOG: RequestBody: '+requestBody);
         system.debug('OCTOPUSH LOG: XML REQUEST BODY: '+req.getBody());
         HTTPResponse res = http.send(req);
         system.debug('OCTOPUSH LOG: XML RESPONSE BODY: '+res.getBody());
         if(estimation)
            smsInputs.put('estimation','true');
         return getOctopushResponse(res, requestBody, smsInputs);
    }
    
    @future(callout=true)
    public static void sendSMSAsync(Boolean estimation, String requestBody, map<String, String> smsInputs){
        globalSmsResult gsr = new globalSmsResult();
        try {
            gsr = sendSMS(estimation, requestBody, smsInputs);
        } catch(Exception e) {
            System.debug('OCTOPUSH LOG: '+e);
        }
    }
    
    public static String getErrorMessageByCode(String code, String isoCode) {
        Map<String, String> errorCodeMapping =  new Map<String, String>();
        if(String.isBlank(isoCode)) isoCode = 'EN';
        if(isoCode.equals('EN')) {
            errorCodeMapping.put('000', Label.ERROR_CODE_000);
            errorCodeMapping.put('100', Label.ERROR_CODE_100);
            errorCodeMapping.put('101', Label.ERROR_CODE_101);
            errorCodeMapping.put('102', Label.ERROR_CODE_102);
            errorCodeMapping.put('103', Label.ERROR_CODE_103);
            errorCodeMapping.put('104', Label.ERROR_CODE_104);
            errorCodeMapping.put('105', Label.ERROR_CODE_105);
            errorCodeMapping.put('106', Label.ERROR_CODE_106);
            errorCodeMapping.put('107', Label.ERROR_CODE_107);
            errorCodeMapping.put('108', Label.ERROR_CODE_108);
            errorCodeMapping.put('109', Label.ERROR_CODE_109);
            errorCodeMapping.put('110', Label.ERROR_CODE_110);
            errorCodeMapping.put('111', Label.ERROR_CODE_111);
            errorCodeMapping.put('112', Label.ERROR_CODE_112);
            errorCodeMapping.put('113', Label.ERROR_CODE_113);
            errorCodeMapping.put('114', Label.ERROR_CODE_114);
            errorCodeMapping.put('115', Label.ERROR_CODE_115);
            errorCodeMapping.put('116', Label.ERROR_CODE_116);
            errorCodeMapping.put('117', Label.ERROR_CODE_117);
            errorCodeMapping.put('118', Label.ERROR_CODE_118);
            errorCodeMapping.put('119', Label.ERROR_CODE_119);
            errorCodeMapping.put('120', Label.ERROR_CODE_120);
            errorCodeMapping.put('121', Label.ERROR_CODE_121);
            errorCodeMapping.put('122', Label.ERROR_CODE_122);
            errorCodeMapping.put('123', Label.ERROR_CODE_123);
            errorCodeMapping.put('124', Label.ERROR_CODE_124);
            errorCodeMapping.put('125', Label.ERROR_CODE_125);
            errorCodeMapping.put('126', Label.ERROR_CODE_126);
            errorCodeMapping.put('127', Label.ERROR_CODE_127);
            errorCodeMapping.put('128', Label.ERROR_CODE_128);
            errorCodeMapping.put('129', Label.ERROR_CODE_129);
            errorCodeMapping.put('130', Label.ERROR_CODE_130);
            errorCodeMapping.put('131', Label.ERROR_CODE_131);
            errorCodeMapping.put('132', Label.ERROR_CODE_132);// Added since 1.15
            errorCodeMapping.put('133', Label.ERROR_CODE_133);// Added since 1.15
            errorCodeMapping.put('150', Label.ERROR_CODE_150);
            errorCodeMapping.put('151', Label.ERROR_CODE_151);
            errorCodeMapping.put('152', Label.ERROR_CODE_152);
            errorCodeMapping.put('153', Label.ERROR_CODE_153);
            errorCodeMapping.put('201', Label.ERROR_CODE_201);
            errorCodeMapping.put('202', Label.ERROR_CODE_202);
            errorCodeMapping.put('203', Label.ERROR_CODE_203);
            errorCodeMapping.put('204', Label.ERROR_CODE_204);
            errorCodeMapping.put('205', Label.ERROR_CODE_205);
            errorCodeMapping.put('206', Label.ERROR_CODE_206);
            errorCodeMapping.put('207', Label.ERROR_CODE_207);
            errorCodeMapping.put('208', Label.ERROR_CODE_208);
            errorCodeMapping.put('209', Label.ERROR_CODE_209);
            errorCodeMapping.put('210', Label.ERROR_CODE_210);
            errorCodeMapping.put('300', Label.ERROR_CODE_300);
            errorCodeMapping.put('301', Label.ERROR_CODE_301);
            errorCodeMapping.put('302', Label.ERROR_CODE_302);
            errorCodeMapping.put('303', Label.ERROR_CODE_303);
            errorCodeMapping.put('304', Label.ERROR_CODE_304);
            errorCodeMapping.put('305', Label.ERROR_CODE_305);
            errorCodeMapping.put('306', Label.ERROR_CODE_306);      
       }
       if(errorCodeMapping.containsKey(code))
			return errorCodeMapping.get(code);
       return 'Octopush server error: '+code+'. Please contact Octopush support.('+Datetime.now()+')';
   }
   
   public class globalSmsResult {
       public list <smsResult> smsResult     {get; set;}
       public Integer totalSms               {get; set;}
       public Integer failureSms             {get; set;}
       public Integer successSms             {get; set;}
       public String httpResBody             {get; set;}
       public String totalCost               {get; set;}
       public String billingCurrency         {get; set;}
       public String estimate_error_code     {get; set;}
       public String templateID              {get; set;}
       public String message                 {get; set;}
       public String receiverType            {get; set;}
       public String sendingDate             {get; set;}
       public Boolean defferedSending        {get; set;}
       public String senderName              {get; set;}
       public String purpose                 {get; set;}
   }
   
   public class smsResult {
        public String error_code     {get; set;}
        public String recipient      {get; set;}
        public String country_code   {get; set;}
        public String cost           {get; set;}
        public String sms_needed     {get; set;}
        public String status         {get; set;}
        public String errorReason    {get; set;}
    }
}