@isTest
global class MockHttpResponseGenerator_TEST implements HttpCalloutMock {
	// Implement this interface method
    global HTTPResponse respond(HTTPRequest req) {
    	// Get request body to simulate response
    	map<String, String> requestParam = new map<String, String>{};
    	for(String aParam : req.getBody().split('&')) {
			list<String> keyVal = aParam.split('=');
			String paramValue = keyVal.size() > 1 ? keyVal[1] : '';
			requestParam.put(keyVal[0], paramValue);
		}
		system.assert(requestParam.containsKey('action') || requestParam.containsKey('sms_recipients'));
		String xmlResponse = '';
		if(requestParam.containsKey('action')) {			
			xmlResponse = estimationSMSCostXmlResponse(requestParam.get('action'));
		} else {
			xmlResponse = sendSMSXmlResponse(requestParam.get('sms_recipients'));
		}
        // Create a fake response
        HttpResponse res = new HttpResponse();
        res.setHeader('Content-Type', 'application/xml');
        res.setBody(xmlResponse);
        res.setStatusCode(200);
        return res;
    }
    
    private String sendSMSXmlResponse(String recipients) {
    	String xmlResponse = '';
    	list<String> multiRecipients = recipients.split(',');
        
        Integer nbMobile = recipients.countMatches('+33');
        // Bulk sending
        if( nbMobile >= 3) {
        	if(recipients.countMatches('+336') == nbMobile) // ALL ok
        		xmlResponse = '<?xmlversion="1.0"encoding="UTF-8"?><octopush><error_code>000</error_code><cost>0.1</cost><balance>0</balance><ticket>XXXSIMULATIONXXX</ticket><sending_date>1433322214</sending_date><number_of_sendings>2</number_of_sendings><currency_code>?</currency_code><successs><success><recipient>'+multiRecipients[0]+'</recipient><country_code>XXX</country_code><cost>0.05</cost><sms_needed>1</sms_needed></success><success><recipient>'+multiRecipients[1]+'</recipient><country_code>XXX</country_code><cost>0.05</cost><sms_needed>1</sms_needed></success></successs><failures/></octopush>';
        	else if(recipients.countMatches('+331') > 0 && recipients.countMatches('+336') > 0) // at least 1 error
        		xmlResponse = '<?xmlversion="1.0"encoding="UTF-8"?><octopush><error_code>000</error_code><cost>0.05</cost><balance>0</balance><ticket>XXXSIMULATIONXXX</ticket><sending_date>1433323625</sending_date><number_of_sendings>1</number_of_sendings><currency_code>?</currency_code><successs><success><recipient>'+multiRecipients[0]+'</recipient><country_code>XXX</country_code><cost>0.05</cost><sms_needed>1</sms_needed></success></successs><failures><failure><error_code>150</error_code><recipient>'+multiRecipients[1]+'</recipient></failure></failures></octopush>';
        	else if(recipients.countMatches('+331') > 0 && recipients.countMatches('+336') == 0) // all errors
        		xmlResponse = '<?xml version="1.0" encoding="UTF-8"?><octopush><error_code>103</error_code></octopush>';	
        }
        // 1 SMS Sent OK
        if(recipients.countMatches('+336') == 1 && recipients.countMatches('+331') == 0)
        	xmlResponse = '<?xmlversion="1.0"encoding="UTF-8"?><octopush><error_code>000</error_code><cost>0.05</cost><balance>0</balance><ticket>XXXSIMULATIONXXX</ticket><sending_date>1433322120</sending_date><number_of_sendings>1</number_of_sendings><currency_code>?</currency_code><successs><success><recipient>'+recipients+'</recipient><country_code>XXX</country_code><cost>0.05</cost><sms_needed>1</sms_needed></success></successs><failures/></octopush>';
        // 2 SMS Sent OK
        else if(recipients.countMatches('+336') > 1 && recipients.countMatches('+331') == 0)
        	xmlResponse = '<?xmlversion="1.0"encoding="UTF-8"?><octopush><error_code>000</error_code><cost>0.1</cost><balance>0</balance><ticket>XXXSIMULATIONXXX</ticket><sending_date>1433322214</sending_date><number_of_sendings>2</number_of_sendings><currency_code>?</currency_code><successs><success><recipient>'+multiRecipients[0]+'</recipient><country_code>XXX</country_code><cost>0.05</cost><sms_needed>1</sms_needed></success><success><recipient>'+multiRecipients[1]+'</recipient><country_code>XXX</country_code><cost>0.05</cost><sms_needed>1</sms_needed></success></successs><failures/></octopush>';
        // 1 SMS OK AND 1 SMS KO
        else if(recipients.countMatches('+336') == 1 && recipients.countMatches('+331') == 1)
        	xmlResponse = '<?xmlversion="1.0"encoding="UTF-8"?><octopush><error_code>000</error_code><cost>0.05</cost><balance>0</balance><ticket>XXXSIMULATIONXXX</ticket><sending_date>1433323625</sending_date><number_of_sendings>1</number_of_sendings><currency_code>?</currency_code><successs><success><recipient>'+multiRecipients[0]+'</recipient><country_code>XXX</country_code><cost>0.05</cost><sms_needed>1</sms_needed></success></successs><failures><failure><error_code>150</error_code><recipient>'+multiRecipients[1]+'</recipient></failure></failures></octopush>';
        // 1 OR 2 SMS Sent KO
        else if(recipients.countMatches('+331') >= 1 && recipients.countMatches('+336') == 0)
        	xmlResponse = '<?xml version="1.0" encoding="UTF-8"?><octopush><error_code>103</error_code></octopush>';
        	
    	return xmlResponse;
    }
    
    private String estimationSMSCostXmlResponse(String action) {
    	String xmlResponse = '';
    	if(action == 'status') {
    		xmlResponse ='<?xml version="1.0" encoding="UTF-8"?><octopush><error_code>000</error_code><cost>1.0</cost><number_of_sendings>1</number_of_sendings></octopush>';
    	} else if(action == 'send') {
    		xmlResponse = '<?xmlversion="1.0"encoding="UTF-8"?><octopush><error_code>000</error_code><cost>0.05</cost><balance>0</balance><ticket>XXXSIMULATIONXXX</ticket><sending_date>1433322120</sending_date><number_of_sendings>1</number_of_sendings><currency_code>?</currency_code><successs><success><recipient>+33600000009</recipient><country_code>XXX</country_code><cost>0.05</cost><sms_needed>1</sms_needed></success></successs><failures/></octopush>';
    	}
    	return xmlResponse;
    }
}