/*
    Utils class for test classes
*/
@isTest
public class Octopush_UtilsTesting {
	public static list<Lead> leads_All;
	public static list<Contact> contacts_All;
	public static list<Campaign> campaigns_All;
	public static list<SMS_Template__c> templates_All;
	public static Octopush_Settings__c Octopush_Settings;
	
	/* Create 5 leads in the system*/
	 public static void initTestingContext_Leads() {
	 	if(leads_All == null) {
		 	leads_All = new list<Lead>{
		 		new Lead(LastName = 'lead1LastName',Company = 'lead1Company',mobilePhone = '+33600000001'),
	            new Lead(LastName = 'lead2LastName',Company = 'lead2Company',mobilePhone = '+33600000002'),
	            new Lead(LastName = 'lead5LastName',Company = 'lead5Company',mobilePhone = '+331wrongNum'),
	            new Lead(LastName = 'lead6LastName',Company = 'lead6Company',mobilePhone = '+331wrongNum'),
	            new Lead(LastName = 'lead7LastName',Company = 'lead7Company',mobilePhone = null)
		 	}; insert leads_All;
	 	}
	 }
	 
	 /* Create 5 Contacts in the system*/
	 public static void initTestingContext_Contacts() {
	 	if(contacts_All == null) {
		 	contacts_All = new list<Contact>{
		 		new Contact(LastName = 'contac1LastName',mobilePhone = '+33600000001'),
	            new Contact(LastName = 'contact2LastName',mobilePhone = '+33600000002'),
	            new Contact(LastName = 'contact5LastName',mobilePhone = '+331wrongNum'),
	            new Contact(LastName = 'contact6LastName',mobilePhone = '+331wrongNum'),
	            new Contact(LastName = 'contact7LastName',mobilePhone = null)
		 	}; insert contacts_All;
		}
	 }
	 
	 /* Create 4 campaigns in the system*/
	 public static void initTestingContext_Campaigns() {
	 	initTestingContext_Contacts();
	 	initTestingContext_Leads();
	 	if(campaigns_All == null) {
		 	campaigns_All = new list<Campaign>{
		 		new Campaign (Name = 'campaignWithLeadsOnly'),
	            new Campaign (Name = 'campaignWithContactsOnly'),
	            new Campaign (Name = 'campaignWithLeadsContacts'),
	            new Campaign (Name = 'campaignWithoutMembersOnly')
		 	}; insert campaigns_All;
			// Add some members in campaign
	        list<CampaignMember> cm = new list<CampaignMember> {
	        	// campaignWithLeadsOnly
	        	New CampaignMember(CampaignId=campaigns_All[0].Id, LeadId= leads_All[2].Id , Status='Opt-In'),//wrong number
	        	New CampaignMember(CampaignId=campaigns_All[0].Id, LeadId= leads_All[3].Id , Status='Opt-In'),//wrong number
	        	// campaignWithContactsOnly
	        	New CampaignMember(CampaignId=campaigns_All[1].Id, ContactId= contacts_All[0].Id , Status='Opt-In'),
	        	New CampaignMember(CampaignId=campaigns_All[1].Id, ContactId= contacts_All[2].Id , Status='Opt-In'),//wrong number
	        	// campaignWithLeadsContacts
	        	New CampaignMember(CampaignId=campaigns_All[2].Id, LeadId= leads_All[0].Id , Status='Opt-In'),
	        	New CampaignMember(CampaignId=campaigns_All[2].Id, ContactId= contacts_All[1].Id , Status='Opt-In')
	        }; insert cm;
	 	}
	}
	
	/* Initialisation of Octopush settings*/
	public static void initTestingContext_OctopushSettings() {
		if(Octopush_Settings == null) {
			Octopush_Settings = new Octopush_Settings__c(
				name='OctopushSettings',
				API_KEY__c='test',
				User_Name__c='test@test.com',
				Default_Sender_Name__c='test',
				History_Recording__c=true,
				Real_SMS_Sending__c=false
			);	insert Octopush_Settings;
		}
	}
	
	/* Initialisation of Octopush template*/
	public static void initTestingContext_SMSTemplate() {
		if(templates_All == null) {
			templates_All = new list<SMS_Template__c>{
				new SMS_Template__c(Message__c='Template1',Sender_Name__c='test'),
				new SMS_Template__c(Message__c='Template2{!contact.name}{!lead.name}{!user.name}',Sender_Name__c='test')
			}; insert templates_All;
		}
	}

	/* Utils methods*/
	public static boolean messageContainsValue(String testValue) {
		Boolean hasError = false;
        for(Apexpages.Message msg: ApexPages.getMessages()){
		    if (msg.getDetail().contains(testValue)) 
	    		hasError = true;
		}
		return hasError;
	}
}