global with sharing class Utils_Octopush {

    public static void showInTestMode() {
        Map<String, Octopush_Settings__c> octoSet = Octopush_Settings__c.getAll();
        if(octoSet.containsKey('OctopushSettings')) {
            Octopush_Settings__c os = octoSet.get('OctopushSettings');
            if( ! os.Real_SMS_Sending__c)
                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_SIMULATION_MODE));       
        }
    }
    
    private static Octopush_Settings__c OctoSettingsInstance = null;
    public static Octopush_Settings__c getOctopushSettings() {  
        if(OctoSettingsInstance == null) {
            List<Octopush_Settings__c> OctopushSettings = Octopush_Settings__c.getall().values();
            if(! OctopushSettings.isEmpty()) {          
                OctoSettingsInstance = OctopushSettings[0];
            }
        }
        return OctoSettingsInstance;
    }
    
    public static string buildOctopushParams(String templateUnMerged, list<sObject> recipients, String addiMobiles, String mobileField) {
        Utils_Octopush.publiPostage pp = Utils_Octopush.mergeFields(templateUnMerged, recipients, addiMobiles, mobileField);
        
        String param = 'sms_recipients=';
        for(String aMobile : pp.mobileList) { param += aMobile+','; } if(! pp.mobileList.isEmpty()) param = param.removeEnd(','); param += '&';
        if(! pp.merge1.isEmpty()) {
            param += 'recipients_first_names=';
            for(String aMerge : pp.merge1) { param += aMerge+','; }
            param = param.removeEnd(','); param += '&';
            // Fix bug when only 1 merge field
            if(pp.merge2.isEmpty()) {
	            param += 'recipients_last_names=';
	            for(String aMerge : pp.merge1) { param += ','; }
	            param = param.removeEnd(','); param += '&';
        	} 
        } 
        if(! pp.merge2.isEmpty()) {
            param += 'recipients_last_names=';
            for(String aMerge : pp.merge2) { param += aMerge+','; }
            param = param.removeEnd(','); param += '&';
        } 
        if(! pp.merge3.isEmpty()) {
            param += 'sms_fields_1=';
            for(String aMerge : pp.merge3) { param += aMerge+','; }
            param = param.removeEnd(','); param += '&';
        } 
        if(! pp.merge4.isEmpty()) {
            param += 'sms_fields_2=';
            for(String aMerge : pp.merge4) { param += aMerge+','; }
            param = param.removeEnd(','); param += '&';
        } 
        if(! pp.merge5.isEmpty()) {
            param += 'sms_fields_3=';
            for(String aMerge : pp.merge5) { param += aMerge+','; }
            param = param.removeEnd(','); param += '&';
        } 
        param += 'sms_text='+pp.smsBodyOcto+'&';
        
        return param;
    }
    
     public static publiPostage mergeFields(String templateUnMerged, list<sObject> recipients, String addiMobiles, String mobileField) {
        set<ID> recipientsIDs = new set<ID>();  
        for(sObject o : recipients) {
            recipientsIDs.add(o.id);
        }
        
        list <String> addiMobilesList = new list<String>();
        if(String.isNotBlank(addiMobiles))
            addiMobilesList = addiMobiles.split(',');
        
        String tempOctopushed = templateUnMerged;
        list<String> SFmergeOrder = new list<String>();
        map<String, map<String, String>> ids2fieldValues = new map<String, map<String, String>>(); 
        if(Utils_Octopush.hasUnmergeFields(templateUnMerged)) { // Publipostage necessary because dynamic template
            map<String, set<String>> mergeObject2Fields = new  map<String, set<String>>();
            // Replace merge field to Octopush format
            Pattern regex = Pattern.compile('\\{!([^}]*)\\}');
            Matcher regexMatcher = regex.matcher(templateUnMerged);
            list<String> tokens = new list<String>();       
            Integer i = 0;   
            while (regexMatcher.find()) {
                if(i < 5) {              
                    String SFMergeField = regexMatcher.group();
                    if(i == 0)
                        tempOctopushed = tempOctopushed.replace(SFMergeField, '{nom}');
                    if(i == 1)
                        tempOctopushed = tempOctopushed.replace(SFMergeField, '{prenom}');  
                    if(i == 2)
                        tempOctopushed = tempOctopushed.replace(SFMergeField, '{ch1}'); 
                    if(i == 3)
                        tempOctopushed = tempOctopushed.replace(SFMergeField, '{ch2}'); 
                    if(i == 4)
                        tempOctopushed = tempOctopushed.replace(SFMergeField, '{ch3}');             
                    i++;                
                    String SFMergeClean = SFMergeField.substring(2,SFMergeField.length()-1).toLowerCase();
                    SFmergeOrder.add(SFMergeClean);
                    
                    String sObjectName = ! SFMergeClean.split('\\.').isEmpty() ? SFMergeClean.split('\\.')[0] : null;
                    String fieldName = ! SFMergeClean.split('\\.').isEmpty() ? SFMergeClean.split('\\.')[1] : null;
                    if(String.isNotBlank(sObjectName)) {
                        sObjectName = sObjectName.toLowerCase();
                        if(! mergeObject2Fields.containsKey(sObjectName))
                            mergeObject2Fields.put(sObjectName, new set<String>{fieldName});
                        else {
                            set<String> temp = mergeObject2Fields.get(sObjectName);
                            temp.add(fieldName);
                            mergeObject2Fields.put(sObjectName, temp);
                        }
                    }
                }
            }
            
            // Check CRUD and remove object that user dont have access to
            List<String> sObjectList = new List<String>(mergeObject2Fields.keySet());			
			Schema.DescribeSobjectResult[] descSObject = Schema.describeSObjects(sObjectList);
            
            for(Schema.DescribeSObjectResult aDescSObject : descSObject) {
            	if(aDescSObject.isAccessible() == false)
            		mergeObject2Fields.remove(aDescSObject.getLabel().toLowerCase());
            		Schema.getGlobalDescribe().get(aDescSObject.getLabel()).getDescribe().fields.getMap();
            }
			
            // Build SOQL queries
            sObject currentUser = null;
            list<sObject> objs = new list<sObject>();
            for(String sObjectName : mergeObject2Fields.keySet()) {
            	Map<String,Schema.SObjectField> m = Schema.getGlobalDescribe().get(sObjectName).getDescribe().fields.getMap();
            	
            	String qry = 'Select id,';
            	for (String fieldToCheck : mergeObject2Fields.get(sObjectName)) {
        			// Check if the user has read access on the each field
        			if (m.containsKey(fieldToCheck) && m.get(fieldToCheck).getDescribe().isAccessible()) {
						qry += fieldToCheck + ',';
        			}
            	}
                qry = qry.removeEnd(','); 
                set<ID> allIds = new set<ID>(recipientsIDs);
                allIds.add(UserInfo.getUserId());
                qry += ' From ' + sObjectName + ' Where ID IN :allIds'; 
                try {
                    list<sObject> temp = Database.query(qry);
                     
                    if(sObjectName.equalsIgnoreCase('user')) {
                        currentUser = temp[0];
                    } else {
                        objs.addAll(temp);
                    }
                } catch(Exception e) {
                	System.debug('OCTOPUSH LOG: Exception - Invalid Octopush template merge fields '+e );
                }        
            }
            
            // Get all values by IDs
            
            if(!objs.isEmpty()) {           
              for(sObject o : objs) {
                String objType = o.id.getSobjectType().getDescribe().getName(); 
                for(String sObjType: mergeObject2Fields.keyset()) {
                    if(objType.equalsIgnoreCase(sObjType) || sObjType.equalsIgnoreCase('user')) {
                        for(String sObjFields: mergeObject2Fields.get(sObjType)) {                      
                            String val = sObjType.equalsIgnoreCase('user') ? getValue(currentUser, sObjFields) : getMergeValue(o, sObjFields, mergeObject2Fields);
                            if(! ids2fieldValues.containsKey(o.id))
                                ids2fieldValues.put(o.id, new map<String, String>{sObjType+'.'+sObjFields=>val});
                            else {
                                map<String, String> temp = ids2fieldValues.get(o.id);
                                temp.put(sObjType+'.'+sObjFields, val);
                                ids2fieldValues.put(o.id, temp);
                            }
                        }
                    } 
                 }                      
              }
            }
        }
            // Fill all list
            publiPostage pp = new publiPostage();
            pp.smsBodyOcto = tempOctopushed;
            pp.SFunmerged = templateUnMerged;
            for (sObject o : recipients) {
                String currentId = String.valueOf(o.get('ID'));
                for (Integer j = 0; j < SFmergeOrder.size(); j++) {
                    String SFMergeClean = SFmergeOrder[j];
                    String val = ids2fieldValues.containsKey(currentId) && ids2fieldValues.get(currentId).containsKey(SFMergeClean) ? ids2fieldValues.get(currentId).get(SFMergeClean) : '';
                    if(j == 0)
                        pp.merge1.add(val);
                    if(j == 1)
                        pp.merge2.add(val);
                    if(j == 2)
                        pp.merge3.add(val); 
                    if(j == 3)
                        pp.merge4.add(val); 
                    if(j == 4)
                        pp.merge5.add(val);
                }
                pp.mobileList.add(String.valueOf(o.get(mobileField)));
            }
            // Additional mobile phones
            for (string aAddMobile : addiMobilesList) {
                pp.mobileList.add(aAddMobile);
                if(! pp.merge1.isEmpty())
                    pp.merge1.add('');
                if(! pp.merge2.isEmpty())
                    pp.merge2.add('');
                if(! pp.merge3.isEmpty())
                    pp.merge3.add('');
                if(! pp.merge4.isEmpty())
                    pp.merge4.add('');
                if(! pp.merge5.isEmpty())
                    pp.merge5.add('');          
            }
        return pp;
    }

    public static String buildOctopushMergeParamString(publiPostage pp) {
        String sms_recipients = 'sms_recipients=';
        String recipients_first_names = 'recipients_first_names=';
        String recipients_last_names = 'recipients_last_names=';
        String sms_fields_1 = 'sms_fields_1=';
        String sms_fields_2 = 'sms_fields_2=';
        String sms_fields_3 = 'sms_fields_3=';
        
        for (String str : pp.mobileList)
            sms_recipients += str+',';
        for (String str : pp.merge1)
            recipients_first_names += str+',';
        for (String str : pp.merge2)
            recipients_last_names += str+',';   
        for (String str : pp.merge3)
            sms_fields_1 += str+',';
        for (String str : pp.merge4)
            sms_fields_2 += str+',';    
        for (String str : pp.merge5)
            sms_fields_3 += str+',';
        
        sms_recipients = sms_recipients.removeEnd(',');
        recipients_first_names = recipients_first_names.removeEnd(',');
        recipients_last_names = recipients_last_names.removeEnd(',');
        sms_fields_1 = sms_fields_1.removeEnd(',');
        sms_fields_2 = sms_fields_2.removeEnd(',');
        sms_fields_3 = sms_fields_3.removeEnd(',');
                        
        return sms_recipients+'&'+recipients_first_names+'&'+recipients_last_names+'&'+sms_fields_1+'&'+sms_fields_2+'&'+sms_fields_3+'&';
    }
      
    public static Boolean hasUnmergeFields(String smsTemplate) {
        Pattern regex = Pattern.compile('\\{!([^}]*)\\}');
        Matcher regexMatcher = regex.matcher(smsTemplate);
        Integer nbOfUnmarged = 0;        
        while (regexMatcher.find()) {
            nbOfUnmarged++;            
        }        
        return nbOfUnmarged>0;
    }
    
    public static mergeResult replaceMergeFields(String smsTemplate, String recordId) {
        mergeResult mr = new mergeResult();
        mr.smsTemplate = smsTemplate;
        mr = Utils_Octopush.replaceMergeFields(mr,'Contact', String.valueOf(recordId));
        mr = Utils_Octopush.replaceMergeFields(mr,'Lead', String.valueOf(recordId));
        mr = Utils_Octopush.replaceMergeFields(mr,'User', UserInfo.getUserId());
        // Remove remaining invalid merge fields
        mr = clearInvalidMergeField( mr);
        return mr;
    }
    
    private static mergeResult clearInvalidMergeField(mergeResult mr) {
        Pattern regex = Pattern.compile('\\{!([^}]*)\\}');
        Matcher regexMatcher = regex.matcher(mr.smsTemplate);
        while (regexMatcher.find()) {
            String dirtyVariable = regexMatcher.group();
            mr.smsTemplate = mr.smsTemplate.replace(dirtyVariable, '');
        }
        return mr;
    }
    
    private static mergeResult replaceMergeFields(mergeResult mr, String objectName, String recordId) {
        objectName = objectName.toLowerCase();
        Pattern regex = Pattern.compile('\\{!([^}]*)\\}');
        Matcher regexMatcher = regex.matcher(mr.smsTemplate);
        Set<String> tokens = new Set<String>();   
            
        while (regexMatcher.find()) {
            String dirtyVariable = regexMatcher.group();
            String variable = dirtyVariable.substring(2,dirtyVariable.length()-1);
            mr.smsTemplate = mr.smsTemplate.replace(variable, variable.toLowerCase());
            variable = variable.toLowerCase(); 
            if(variable.startsWith(objectName + '.')){                
                tokens.add(variable.replace(objectName + '.', ''));
            }
        }
        if(tokens.size() > 0) { // Process only dynamic templates            
            // Check CRUD and remove object that user dont have access to
            List<String> sObjectList = new List<String>{objectName};			
			Schema.DescribeSobjectResult[] descSObject = Schema.describeSObjects(sObjectList);
			Map<String,Schema.SObjectField> m = Schema.getGlobalDescribe().get(objectName).getDescribe().fields.getMap();    
            
            String qry = 'Select ';
            for(String s : tokens){
            	if (m.containsKey(s) && m.get(s).getDescribe().isAccessible()) // Check user read access on each fields
                	qry += s + ',';
            }            
            qry = qry.removeEnd(','); 
            try {
                SObject o = Database.query(qry + ' From ' + objectName + ' Where ID = :recordID');                
                for(String s : tokens){                     
                    mr.smsTemplate = mr.smsTemplate.replace('{!' + objectName + '.' + s + '}', getValue(o, s));
                }
            } catch(Exception e) { 
                 mr.error = true;           
            }
        }
        return mr;
    }
    
     private static String getMergeValue(SObject o, String fieldName, map<String, set<String>> mergeObject2Fields){
        try{
            return (o.get(fieldName) != null) ? String.valueOf(o.get(fieldName)) : '';
        }catch(Exception e) {return '';}
    }
    
    private static String getValue(SObject o, String fieldName){
        return (o.get(fieldName) != null) ? String.valueOf(o.get(fieldName)) : '';
    }
    
    
     public static String convertToOctopushDate(DateTime inputDateTime) {
       if(inputDateTime == null)
           return null;
        String  dateTimeToGMT1 = inputDateTime.format('yyyy-MM-dd\'T\'HH:mm:ss.SSS\'Z\'', 'GMT+01:00');       
        return dateTimeToGMT1;
   }
   
   public static String getReceiverType(Id originID) {
        if(originID == null)
            return 'Anonymous';
        if(originID.getSobjectType() == Schema.Contact.SObjectType)
            return 'Contact';
        if(originID.getSobjectType() == Schema.Lead.SObjectType)
            return 'Lead';
        if(originID.getSobjectType() == Schema.Campaign.SObjectType)
            return 'Campaign';  
        return 'Anonymous';
   }
   
   public class publiPostage {
        public publiPostage() {
            smsBodyOcto = null;
            SFunmerged = null;
            mobileList = new list<String>();
            merge1 = new list<String>();
            merge2 = new list<String>();
            merge3 = new list<String>();
            merge4 = new list<String>();
            merge5 = new list<String>();
        }
        public String smsBodyOcto       {get; set;}
        public String SFunmerged        {get; set;}
        public list<String> mobileList  {get; set;}
        public list<String> merge1      {get; set;}
        public list<String> merge2      {get; set;}
        public list<String> merge3      {get; set;}
        public list<String> merge4      {get; set;}
        public list<String> merge5      {get; set;}                        
    }
    
    public class mergeResult {
        public String smsTemplate       {get; set;}
        public Boolean error            {get; set;}
        public mergeResult() {
            smsTemplate = ''; error = false;
        }
    }
}