/*
    Testing class for VF_CampaignBulkSMS_CTRL page
*/
@isTest
public class VF_CampaignBulkSMS_CTRL_TEST {
// Init data for testing
    static {
        Octopush_UtilsTesting.initTestingContext_Leads();
        Octopush_UtilsTesting.initTestingContext_Contacts();
        Octopush_UtilsTesting.initTestingContext_Campaigns();
        // Set mock callout class
		Test.setMock(HttpCalloutMock.class, new MockHttpResponseGenerator_TEST());
    }
	
     public static testMethod void T1_Constructors() {
        Test.startTest();          	
            // Octopush settings not set
            VF_CampaignBulkSMS_CTRL ctrl = new VF_CampaignBulkSMS_CTRL();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.WARNING_MISSING_OCTOPUSH_SETTINGS));			
			// Octopush settings set + none campaign selected		
            Octopush_UtilsTesting.initTestingContext_OctopushSettings();
            PageReference pageRef = Page.VF_CampaignBulkSMS;      				
			Test.setCurrentPage(pageRef);
			pageRef.getParameters().put('retURL', '/home/home.jsp');
            VF_CampaignBulkSMS_CTRL ctrl2 = new VF_CampaignBulkSMS_CTRL();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_INVALID_CAMPAIGN));
            ctrl2.doCancel();
            // Octopush settings set + valid campaign with no members
			pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.campaigns_All[3].Id));
			Test.setCurrentPage(pageRef);
            VF_CampaignBulkSMS_CTRL ctrl3 = new VF_CampaignBulkSMS_CTRL();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_CAMPAIGN_HAS_NO_MEMBERS));
            // Octopush settings set + valid campaign with members
			pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.campaigns_All[2].Id));
			Test.setCurrentPage(pageRef);
            VF_CampaignBulkSMS_CTRL ctrl4 = new VF_CampaignBulkSMS_CTRL();
        Test.stopTest();
    }

     public static testMethod void T2_PositiveSendingSomeSMSError() {
     	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Octopush_UtilsTesting.initTestingContext_SMSTemplate();
        Test.startTest();
        	PageReference pageRef = Page.VF_CampaignBulkSMS;
			pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.campaigns_All[1].Id));
			Test.setCurrentPage(pageRef);
            // Sending with some SMS in error
            VF_CampaignBulkSMS_CTRL ctrl = new VF_CampaignBulkSMS_CTRL();
            ctrl.getItems();
            ctrl.isDefferedSending=true;
            ctrl.aSMS.Sending_Date__c = (DateTime.now()+ (1/6.0));
            ctrl.aSMS.Sender_Name__c='test';
            ctrl.aSMS.SMS_Message__c='test';
			ctrl.aSMS.SMS_Template__c = Octopush_UtilsTesting.templates_All[1].id;
            ctrl.fillTemplateContent();
            ctrl.showSendingMsg();
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.WARNING_SOME_SMS_COULD_NOT_BE_SENT));
        Test.stopTest();
    	system.assertEquals([SELECT id FROM SMS_History__c].size(), 2);
    	system.assertEquals([SELECT Status__c FROM SMS_History__c WHERE Status__c = 'Error'].size(), 1);
    }

    public static testMethod void T3_PositiveSendingNoError() {
     	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
        	PageReference pageRef = Page.VF_CampaignBulkSMS;
			pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.campaigns_All[2].Id));
			Test.setCurrentPage(pageRef);
            // Sending with some SMS in error
            VF_CampaignBulkSMS_CTRL ctrl = new VF_CampaignBulkSMS_CTRL();
            ctrl.aSMS.Sender_Name__c='test';
            ctrl.aSMS.SMS_Message__c='test';
            ctrl.showSendingMsg();
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.CONFIRM_TOTAL_SMS_SENT));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 2);
    	system.assertEquals([SELECT Status__c FROM SMS_History__c WHERE Status__c = 'Error'].size(), 0);
    }

    public static testMethod void T4_PositiveSendingAllInError() {
     	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
        	PageReference pageRef = Page.VF_CampaignBulkSMS;
			pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.campaigns_All[0].Id));
			Test.setCurrentPage(pageRef);
            // Sending with some SMS in error
            VF_CampaignBulkSMS_CTRL ctrl = new VF_CampaignBulkSMS_CTRL();
            ctrl.aSMS.SMS_Message__c='test';
            ctrl.showSendingMsg();
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_SMS_FAILED_TO_BE_SENT));
        Test.stopTest();
    }

    public static testMethod void T5_MandatoryChecking() {
     	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
        	PageReference pageRef = Page.VF_CampaignBulkSMS;
			pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.campaigns_All[0].Id));
			Test.setCurrentPage(pageRef);
			VF_CampaignBulkSMS_CTRL ctrl = new VF_CampaignBulkSMS_CTRL();
            // Empty mandatory fields
            ctrl.aSMS.Sender_Name__c='';
            ctrl.aSMS.SMS_Message__c='';
            ctrl.isDefferedSending=true;
            ctrl.showSendingMsg();
            ctrl.showSendingResultMsg();
            ctrl.aSMS.Sending_Date__c = (DateTime.now()- (1/6.0));
            ctrl.showSendingMsg();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_MISSING_MESSAGE));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 0);
    }

	public static testMethod void T6_MandatoryFieldEstimationSending() {
        Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
        	PageReference pageRef = Page.VF_CampaignBulkSMS;
        	pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.campaigns_All[0].Id));
			Test.setCurrentPage(pageRef);
            VF_CampaignBulkSMS_CTRL ctrl = new VF_CampaignBulkSMS_CTRL();
            ctrl.showEstimatingMsg();
            ctrl.estimateSMS();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_MISSING_MESSAGE));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 0);
    }

    public static testMethod void T7_PositiveEstimationSending() {
        Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
        	PageReference pageRef = Page.VF_CampaignBulkSMS;
        	pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.campaigns_All[2].Id));
			Test.setCurrentPage(pageRef);
            VF_CampaignBulkSMS_CTRL ctrl = new VF_CampaignBulkSMS_CTRL();
            ctrl.isDefferedSending=true;
            ctrl.aSMS.Sending_Date__c = (DateTime.now()+ (1/6.0));
            ctrl.aSMS.SMS_Message__c='test';
            ctrl.showEstimatingMsg();
            ctrl.estimateSMS();
            ctrl.estimationStatus();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.CONFIRM_ESTIMATION_COST_COMPLETED));
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.CONFIRM_TOTAL_SMS_SENT));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 2);
    }
}