/*
    Testing class for VF_ContactBulkSMS_CTR page
*/
@isTest
public class VF_ContactBulkSMS_CTRL_TEST {
    // Init data for testing
    static {
        Octopush_UtilsTesting.initTestingContext_Contacts();
        Octopush_UtilsTesting.initTestingContext_Leads();
        // Set mock callout class
        Test.setMock(HttpCalloutMock.class, new MockHttpResponseGenerator_TEST());
        PageReference pageRef = Page.VF_ContactBulkSMS;
        Test.setCurrentPage(pageRef);
    }

     public static testMethod void T1_Constructors() {
        Test.startTest();
            // Octopush settings not set
            ApexPages.StandardSetController sc = new ApexPages.standardSetController(Octopush_UtilsTesting.contacts_All);
            sc.setSelected(new list<Contact>{Octopush_UtilsTesting.contacts_All[0]});
            VF_ContactBulkSMS_CTRL ctrl = new VF_ContactBulkSMS_CTRL(sc);
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.WARNING_MISSING_OCTOPUSH_SETTINGS));         
            
            Octopush_UtilsTesting.initTestingContext_OctopushSettings();
            // Octopush settings set + none contact selected
            ApexPages.StandardSetController sc2 = new ApexPages.standardSetController(Octopush_UtilsTesting.contacts_All);
            sc2.setSelected(new list<Contact>());
            VF_ContactBulkSMS_CTRL ctrl2 = new VF_ContactBulkSMS_CTRL(sc2);
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_NO_RECORDS_SELECTED));
            
            // Octopush settings set + select all contacts (1 has no mobile phone set)
            ApexPages.StandardSetController sc3 = new ApexPages.standardSetController(Octopush_UtilsTesting.contacts_All);
            sc3.setSelected(Octopush_UtilsTesting.contacts_All);
            VF_ContactBulkSMS_CTRL ctrl3 = new VF_ContactBulkSMS_CTRL(sc3);
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.WARNING_MISSING_MOBILE_FOR_SOME_RECORDS));
        Test.stopTest();
    }

     public static testMethod void T2_PositiveSendingSomeSMSError() {
        Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Octopush_UtilsTesting.initTestingContext_SMSTemplate();
        Test.startTest();
            // Sending with some SMS in error
            ApexPages.StandardSetController sc = new ApexPages.standardSetController(Octopush_UtilsTesting.contacts_All);
            sc.setSelected(Octopush_UtilsTesting.contacts_All);
            VF_ContactBulkSMS_CTRL ctrl = new VF_ContactBulkSMS_CTRL(sc);
            ctrl.getItems();
            ctrl.isDefferedSending=true;
            ctrl.aSMS.Sending_Date__c = (DateTime.now()+ (1/6.0));
            ctrl.aSMS.Sender_Name__c='test';
            ctrl.aSMS.SMS_Message__c='test';
            ctrl.aSMS.SMS_Template__c = Octopush_UtilsTesting.templates_All[1].id;
            ctrl.fillTemplateContent();
            ctrl.showSendingMsg();
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.WARNING_SOME_SMS_COULD_NOT_BE_SENT));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 2);
        system.assertEquals([SELECT Status__c FROM SMS_History__c WHERE Status__c = 'Error'].size(), 1);
    }

    public static testMethod void T3_PositiveSendingNoError() {
        Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
             // Sending SMS with no errors
            ApexPages.StandardSetController sc2 = new ApexPages.standardSetController(Octopush_UtilsTesting.contacts_All);
            sc2.setSelected(new list<Contact>{Octopush_UtilsTesting.contacts_All[0], Octopush_UtilsTesting.contacts_All[1]});
            VF_ContactBulkSMS_CTRL ctrl2 = new VF_ContactBulkSMS_CTRL(sc2);
            ctrl2.aSMS.Sender_Name__c='test';
            ctrl2.aSMS.SMS_Message__c='test';
            ctrl2.showSendingMsg();
            ctrl2.sendSMS();
            ctrl2.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.CONFIRM_TOTAL_SMS_SENT));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 2);
        system.assertEquals([SELECT Status__c FROM SMS_History__c WHERE Status__c = 'Error'].size(), 0);
    }

    public static testMethod void T4_PositiveSendingAllInError() {
        Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
           // Sending with contact with no mobile => error
            ApexPages.StandardSetController sc3 = new ApexPages.standardSetController(Octopush_UtilsTesting.contacts_All);
            sc3.setSelected(new list<Contact>{Octopush_UtilsTesting.contacts_All[2], Octopush_UtilsTesting.contacts_All[3]});
            VF_ContactBulkSMS_CTRL ctrl3 = new VF_ContactBulkSMS_CTRL(sc3);
            ctrl3.aSMS.Sender_Name__c='test';
            ctrl3.aSMS.SMS_Message__c='test';
            ctrl3.showSendingMsg();
            ctrl3.sendSMS();
            ctrl3.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_SMS_FAILED_TO_BE_SENT));
        Test.stopTest();
    }

    public static testMethod void T5_MandatoryChecking() {
        Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
            // Sending with some SMS in error
            ApexPages.StandardSetController sc = new ApexPages.standardSetController(Octopush_UtilsTesting.contacts_All);
            sc.setSelected(Octopush_UtilsTesting.contacts_All);
            VF_ContactBulkSMS_CTRL ctrl = new VF_ContactBulkSMS_CTRL(sc);
            ctrl.aSMS.Sender_Name__c='';
            ctrl.aSMS.SMS_Message__c='';
            ctrl.isDefferedSending=true;
            ctrl.showSendingMsg();
            ctrl.showSendingResultMsg();
            ctrl.aSMS.Sending_Date__c = (DateTime.now()- (1/6.0));
            ctrl.showSendingMsg();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_MISSING_MESSAGE));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 0);
    }

    public static testMethod void T6_MandatoryFieldEstimationSending() {
        Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
            ApexPages.StandardSetController sc = new ApexPages.standardSetController(Octopush_UtilsTesting.contacts_All);
            sc.setSelected(Octopush_UtilsTesting.contacts_All);
            VF_ContactBulkSMS_CTRL ctrl = new VF_ContactBulkSMS_CTRL(sc);
            ctrl.showEstimatingMsg();
            ctrl.estimateSMS();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_MISSING_MESSAGE));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 0);
    }

    public static testMethod void T7_PositiveEstimationSending() {
        Octopush_UtilsTesting.initTestingContext_OctopushSettings();
        Test.startTest();
            ApexPages.StandardSetController sc = new ApexPages.standardSetController(Octopush_UtilsTesting.contacts_All);
            sc.setSelected(new list<Contact>{Octopush_UtilsTesting.contacts_All[0], Octopush_UtilsTesting.contacts_All[1]});
            VF_ContactBulkSMS_CTRL ctrl = new VF_ContactBulkSMS_CTRL(sc);
            ctrl.isDefferedSending=true;
            ctrl.aSMS.Sending_Date__c = (DateTime.now()+ (1/6.0));
            ctrl.aSMS.SMS_Message__c='test';
            ctrl.showEstimatingMsg();
            ctrl.estimateSMS();
            ctrl.estimationStatus();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.CONFIRM_ESTIMATION_COST_COMPLETED));
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.CONFIRM_TOTAL_SMS_SENT));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 2);
    }
}