public with sharing class VF_LeadBulkSMS_CTRL {
    
    public Integer total                        {get; set;}
    public Integer nbWithNoMobile               {get; set;}
    public Integer isSMSSent                    {get; set;}
    public Boolean disableSendButton            {get; set;}
    public Boolean disableEstimationButton      {get; set;}
    public Boolean largeCampaign                {get; set;}
    
    public Boolean isDefferedSending            {get; set;}
    public String additionalMobile              {get; set;}
    public SMS_History__c aSMS                  {get; set;}
    
    private ApexPages.StandardSetController standardController;
    private list<Lead> validLeads;
    private Boolean hasUIValidationError;
    private list<API_Octopush.globalSmsResult> gsrList;
    private API_Octopush.globalSmsResult gsr = new API_Octopush.globalSmsResult();  
    private String userID;

    public VF_LeadBulkSMS_CTRL(ApexPages.StandardSetController standardController) {
    	this.standardController = standardController;
        aSMS = new SMS_History__c(SMS_Purpose__c='Marketing');
        Octopush_Settings__c os = Utils_Octopush.getOctopushSettings();
        if(os != null) {
            aSMS.sender_Name__c = os.Default_Sender_Name__c;
            aSMS.SMS_Purpose__c = os.SMS_Purpose__c;
            Utils_Octopush.showInTestMode();
            isDefferedSending = hasUIValidationError = disableSendButton = disableEstimationButton = false;         
            nbWithNoMobile = 0;
            List<Lead> sc = standardController.getSelected();
            if (Schema.sObjectType.Lead.isAccessible() && Schema.sObjectType.Lead.fields.MobilePhone.isAccessible()) {
	        	List<Lead> selectedLeads = [select mobilePhone from Lead where id in: sc];
	            total = sc.size();
	            validLeads = new list<Lead>();
				for(Lead selectedLead : selectedLeads) {
	                if(String.isBlank(selectedLead.MobilePhone)) {
	                    nbWithNoMobile++;
	                } else {
	                     validLeads.add(selectedLead);
	                }
	            }
	            largeCampaign = validLeads.size() >= API_Octopush.PART_THRESHOLD;
	            disableSendButton = largeCampaign;
	            // Check if some record have empty mobile phone
	            if(nbWithNoMobile > 0)
	                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.WARNING , Label.WARNING_MISSING_MOBILE_FOR_SOME_RECORDS)); 
	            if(sc.size() == 0) {
	                disableSendButton = true;
	                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR , Label.ERROR_NO_RECORDS_SELECTED));
	            }
            } else {
            	 disableSendButton = disableEstimationButton = true;
            	 ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Insufficent priviliges: You must have at least Read access to Lead object and mobile phone field. Please contact your administrator for support.'));
            }
        } else {
            disableSendButton = disableEstimationButton = true;
            ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR , Label.WARNING_MISSING_OCTOPUSH_SETTINGS)); 
        }
    }
    
     public PageReference fillTemplateContent() {
     	if (Schema.sObjectType.SMS_Template__c.isAccessible() && Schema.sObjectType.SMS_Template__c.fields.Message__c.isAccessible() && Schema.sObjectType.SMS_Template__c.fields.Sender_Name__c.isAccessible()) {
	       list<SMS_Template__c> templateSelected = [SELECT Message__c, Sender_Name__c FROM SMS_Template__c WHERE id =: aSMS.SMS_Template__c limit 1];
	        Utils_Octopush.mergeResult mr = new Utils_Octopush.mergeResult();
	        if(! templateSelected.isEmpty()) {
	            mr = Utils_Octopush.replaceMergeFields(templateSelected[0].Message__c, validLeads[0].id);
	            aSMS.SMS_Message__c = templateSelected[0].Message__c;
	            aSMS.Sender_Name__c = templateSelected[0].Sender_Name__c;
	        }
	        if(Utils_Octopush.hasUnmergeFields(aSMS.SMS_Message__c)) {
	             ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_DYNAMIC_TEMPLATE_USED));
	             ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_DYNAMIC_TEMPLATE_1ST_PERSON_PREVIEW));
	             ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, mr.smsTemplate));
	        }
     	} else {
     		ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Insufficent priviliges: You must have at least Read access to SMS Template object and message and sender name field. Please contact your administrator for support.'));
     	}
        return null;
    }
    
     public PageReference estimateSMS() {
     	if(! Schema.sObjectType.SMS_History__c.isCreateable()) {   
             ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Insufficent priviliges: You must have at least Read/Create rights to the SMS History object. Please contact your administrator for support.'));
             return null; 
         }
         
        list<String> UIerrors = mandatoryChecking();
        if(! UIerrors.isEmpty()) {
            for(String anError : UIerrors)
                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, anError)); 
            return null;
        }
        
        isSMSSent = 0;
        gsrList = new list<API_Octopush.globalSmsResult>();
        //userID = UserInfo.getUserId()+Math.mod(Math.round(Math.random()*1000), 1000);
    	userID = UserInfo.getUserId()+DateTime.now();         
        String requestBody = '';        
        requestBody += 'sms_sender='+aSMS.Sender_Name__c+'&';
        requestBody += 'user_batch_id='+userID+'&';
        if(isDefferedSending) {
            requestBody += 'sending_date='+aSMS.Sending_Date__c+'&';
            requestBody += 'sms_mode=2&';
        }
        // Set sending parts
        Integer nbOfSMS = validLeads.size();
        list<Lead> leadPart = new list<Lead>();
        for (Integer i = 0, k = 0; i < nbOfSMS; k++) {
            String requestBodyPart = requestBody;
            leadPart.clear();
            for (Integer j = 0;j < API_Octopush.PART_SPLITTED_NB && i < nbOfSMS; j++) {                 
                leadPart.add(validLeads[i]);
                i++;
            }
            String octoParams = Utils_Octopush.buildOctopushParams(aSMS.SMS_Message__c, leadPart, ((Boolean)(k==0) ? additionalMobile : null), 'MobilePhone');
            requestBodyPart += octoParams;
            requestBodyPart += 'finished='+((Boolean) (i >= nbOfSMS) ? '1': '0')+'&';
            map<String, String> params = new map<String, String> {'templateID'=>aSMS.SMS_Template__c, 'receiverType'=>'Lead'};
            if(leadPart.size() == 1)
                params.put('recipient', leadPart[0].mobilePhone);
            gsrList.add(API_Octopush.sendSMS(true, requestBodyPart, params));
        }
		isSMSSent = 2;
        
        return null;
    }
    
    
    public PageReference estimationStatus() {
       disableSendButton = true;
        list<String> UIerrors = mandatoryChecking();
        if(UIerrors.size() > 0) {
            for(String anError : UIerrors)
                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, anError)); 
        } else {
	            String smsPurpose = aSMS.SMS_Purpose__c == 'Transactional' ? 'transactional=1&':'';
	            gsr = API_Octopush.getEstimationStatus('status', userID, null, smsPurpose);
	            System.debug('@@@ Octopush logs: Erreur to display: '+gsr.estimate_error_code);
	            if(gsr.estimate_error_code == '000') {
	                disableSendButton = false;
	                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_TOTAL_SENDING_COST+gsr.totalCost)); //+gsr.billingCurrency
	                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_SMS_SUCCESS+gsr.successSms+'/'+total));   
	                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_SMS_IN_FAILURE+(total-gsr.successSms)+'/'+total));
	                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.CONFIRM, Label.CONFIRM_ESTIMATION_COST_COMPLETED));
	                isSMSSent = 3;
	            } else if(gsr.estimate_error_code == '129'){
	                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, API_Octopush.getErrorMessageByCode(gsr.estimate_error_code, 'EN')+' '+Label.INFO_PLEASE_WAIT));
	            } else {                
	                
	                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, API_Octopush.getErrorMessageByCode(gsr.estimate_error_code, 'EN')));
	                // Out of credit case: let user know cost and nb of success sms to be able to buy enought credit
	                if(gsr.estimate_error_code == '104') {
	                	ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_TOTAL_SENDING_COST+gsr.totalCost)); //+gsr.billingCurrency
		                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_SMS_SUCCESS+gsr.successSms+'/'+total));   
		                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_SMS_IN_FAILURE+(total-gsr.successSms)+'/'+total));
	                }
	                isSMSSent = -3;
	            }
        }
        return null;
    }
    
    
    public PageReference sendSMS() {
    	if(! Schema.sObjectType.SMS_History__c.isCreateable()) {   
			ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Insufficent priviliges: You must have at least Read/Create rights to the SMS History object. Please contact your administrator for support.'));
			return null; 
       }
       isSMSSent = 0;
        if(mandatoryChecking().isEmpty()) {
            String smsPurpose = aSMS.SMS_Purpose__c == 'Transactional' ? 'transactional=1&':'';
            // Send after estimation
            if(userID != null) {
                gsr = API_Octopush.getEstimationStatus('send', userID, gsrList, smsPurpose);
            } else { // Send directly if under estimation threshold
                String octoParams = Utils_Octopush.buildOctopushParams(aSMS.SMS_Message__c, validLeads, additionalMobile, 'MobilePhone');
                String requestBody = '';        
                requestBody += 'sms_sender='+aSMS.Sender_Name__c+'&';
                if(isDefferedSending) {
                    requestBody += 'sending_date='+aSMS.Sending_Date__c+'&';
                    requestBody += 'sms_mode=2&';
                }
                requestBody += smsPurpose;
                requestBody += octoParams;
                gsr = API_Octopush.sendSMS(false, requestBody, new map<String, String> {'templateID'=>aSMS.SMS_Template__c, 'receiverType'=>'Lead'});
            }
            if(gsr.totalSms == gsr.successSms) {                   
                isSMSSent = 0;
            } else if(gsr.failureSms > 0 && gsr.successSms > 0) {
                isSMSSent = -1;                            
            } else {
                isSMSSent = -99;                   
            }           
        }
        return null;   
    }
    
    private list<String> mandatoryChecking() {
         hasUIValidationError = false; 
        list <String> errorList = new list<String>();
        
        if(String.isEmpty(aSMS.SMS_Message__c)) {
            errorList.add(Label.ERROR_MISSING_MESSAGE);
            hasUIValidationError = true;          
        }
        if(isDefferedSending && aSMS.Sending_Date__c == null) {
            errorList.add(Label.ERROR_SENDING_DATE_MESSAGE);
            hasUIValidationError = true;           
        }
        if(isDefferedSending && aSMS.Sending_Date__c != null && aSMS.Sending_Date__c < DateTime.now()) {
            errorList.add(Label.ERROR_DEFFERED_DATE_NOT_SET_IN_PAST);
            hasUIValidationError = true; 
        }
        return errorList;
    }
    
     public PageReference showEstimatingMsg() { 
        if(mandatoryChecking().isEmpty()) {  
            disableSendButton = true;
            ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, LABEL.INFO_ESTIMATING_PLEASE_WAIT));
        } 
        return null;
    }
    
    public PageReference showSendingMsg() {
        if(mandatoryChecking().isEmpty()) {   
             ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, LABEL.INFO_SENDING_SMS)); 
         }
         return null;
    }
    
    public PageReference showSendingResultMsg() {
        if(hasUIValidationError) {
            for(String anError : mandatoryChecking())
                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, anError)); 
            return null;
        } else {
             if(isSMSSent == 0) {
                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_TOTAL_SMS_SENT+gsr.successSms+'/'+gsr.totalSms));                
                 ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.CONFIRM, Label.CONFIRM_TOTAL_SMS_SENT));
                    
             } else if(isSMSSent == -1) {
                 ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_TOTAL_SMS_SENT+gsr.successSms+'/'+gsr.totalSms));      
                 ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.WARNING, Label.WARNING_SOME_SMS_COULD_NOT_BE_SENT));
                 for(API_Octopush.smsResult sr : gsr.smsResult) {
                     if(sr.error_code != null)
                         ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.WARNING, sr.recipient+': '+API_Octopush.getErrorMessageByCode(sr.error_code, 'EN')));
                }
                   
            } else if(isSMSSent == -99){
                 ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, Label.ERROR_SMS_FAILED_TO_BE_SENT));
                 for(API_Octopush.smsResult sr : gsr.smsResult) {
                    if(sr.error_code != null)
                        ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, sr.recipient+': '+API_Octopush.getErrorMessageByCode(sr.error_code, 'EN')));
                }
             } else {
                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, Label.ERROR_UNKNOWN_ERROR));
             }
        }
        return null;
    }
    
    public List<SelectOption> getItems() {
        List<SelectOption> options = new List<SelectOption>(); 
        options.add(new SelectOption('Marketing','Marketing SMS')); 
        options.add(new SelectOption('Transactional','Transactional SMS')); return options;         
    }
}