public with sharing class VF_SendSingleSMS_CTRL {   
    
    public SMS_History__c aSMS                      {get; set;}    
    public String additionalMobile                  {get; set;}
    public Boolean isDefferedSending                {get; set;}
    public Integer isSMSSent                        {get; set;}
    public ID originID                              {get; set;}
    public String relatedRecordName                 {get; set;}
    public Boolean isLeadOrContact                  {get; set;}
    public Boolean disableSendButton                {get; set;}
    
    private  API_Octopush.globalSmsResult gsr = new API_Octopush.globalSmsResult();
    private Boolean hasUIValidationError = false;
    
    public VF_SendSingleSMS_CTRL () { 
        aSMS = new SMS_History__c(SMS_Purpose__c='Transactional');
        Octopush_Settings__c os = Utils_Octopush.getOctopushSettings();
        if(os != null) {
            disableSendButton = false;  
            aSMS.sender_Name__c = os.Default_Sender_Name__c;
            aSMS.SMS_Purpose__c = os.SMS_Purpose__c;
            isDefferedSending = isLeadOrContact = false;
            originID = ApexPages.currentPage().getParameters().get('relatedObject');
            relatedRecordName = '';
            if(originID != null) {          
                if(originID.getSObjectType() == Contact.sObjectType) {
                    isLeadOrContact = true;
                    if (Schema.sObjectType.Contact.isAccessible() && Schema.sObjectType.Contact.fields.MobilePhone.isAccessible()) {
	                    list<Contact> contactFound = [SELECT Name, MobilePhone FROM Contact WHERE id =: originID limit 1];
	                    if(contactFound.size() == 0) {
	                    	 ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Invalid contact')); 
	                    	 disableSendButton = true;
	                    } else {
		                    Contact c = contactFound[0];
		                    relatedRecordName = c.name;                
		                    if(c.MobilePhone != null)
		                        aSMS.Mobile_Number__c = c.MobilePhone;
		                    else {
		                        aSMS.Mobile_Number__c = '';
		                        ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_PHONE_NOT_FILLED_FOR_CONTACT)); 
		                    }
	                    }
                    } else {
                    	disableSendButton = true;
                    	ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Insufficent priviliges: You must have at least Read access to Contact object and mobile phone field. Please contact your administrator for support.')); 
                    }                 
                } else if(originID.getSObjectType() == Lead.sObjectType) {
                    isLeadOrContact = true;
                    if (Schema.sObjectType.Lead.isAccessible() && Schema.sObjectType.Lead.fields.MobilePhone.isAccessible()) {
	                    list<Lead> leadFound = [SELECT Name, MobilePhone FROM Lead WHERE id =: originID limit 1];
	                    if(leadFound.size() == 0) {
	                     	ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Invalid lead')); 
	                    	disableSendButton = true;
	                    } else {
	                    	 Lead l = leadFound[0];
		                     relatedRecordName = l.name;
		                    if(l.MobilePhone != null)
		                        aSMS.Mobile_Number__c = l.MobilePhone;
		                    else {
		                        aSMS.Mobile_Number__c = '';
		                        ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_PHONE_NOT_FILLED_FOR_LEAD)); 
		                    }
	                    }
					} else {
						disableSendButton = true;  
                    	ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Insufficent priviliges: You must have at least Read access to Lead object and mobile phone field. Please contact your administrator for support.')); 
                    }   
                }
            }
            Utils_Octopush.showInTestMode();
        } else {
            disableSendButton = true;
            ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR , Label.WARNING_MISSING_OCTOPUSH_SETTINGS)); 
        }
    }
    
    public PageReference fillTemplateContent() {
    	if (Schema.sObjectType.SMS_Template__c.isAccessible() && Schema.sObjectType.SMS_Template__c.fields.Message__c.isAccessible() && Schema.sObjectType.SMS_Template__c.fields.Sender_Name__c.isAccessible()) {
	        list<SMS_Template__c> templateSelected = [SELECT Message__c, Sender_Name__c FROM SMS_Template__c WHERE id =: aSMS.SMS_Template__c limit 1];
	        if(! templateSelected.isEmpty()) {
	            Utils_Octopush.mergeResult mr = new Utils_Octopush.mergeResult();
	            mr = Utils_Octopush.replaceMergeFields(templateSelected[0].Message__c, String.valueOf(originID));
	            aSMS.SMS_Message__c = mr.smsTemplate;
	            aSMS.Sender_Name__c = templateSelected[0].Sender_Name__c;
	            
	            if(mr.error) {
	             ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.WARNING, Label.WARNING_FIELD_NOT_MERGEABLE));
	            }
	        }
    	} else {
    		ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Insufficent priviliges: You must have at least Read access to SMS Template object and message and sender name field. Please contact your administrator for support.'));
    	}
        return null;
    }
    
    public PageReference sendSMS() {
    	if(! Schema.sObjectType.SMS_History__c.isCreateable()) {   
             ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Insufficent priviliges: You must have at least Read/Create rights to the SMS History object. Please contact your administrator for support.'));
             return null; 
         }
        isSMSSent = 0;
        if(mandatoryChecking().isEmpty()) {
            String requestBody = '';        
            requestBody += 'sms_sender='+aSMS.Sender_Name__c+'&';
            if(isDefferedSending) {
                requestBody += 'sending_date='+aSMS.Sending_Date__c+'&';
                requestBody += 'sms_mode=2&';
            }
            requestBody += aSMS.SMS_Purpose__c == 'Transactional' ? 'transactional=1&':'';
            requestBody += 'sms_text='+aSMS.SMS_Message__c+'&';
            String mobiles = aSMS.Mobile_Number__c;
            mobiles = mobiles.removeEnd(',');
            if(String.isNotBlank(additionalMobile)) {
                mobiles += ','+ additionalMobile;
                mobiles = mobiles.removeEnd(',');
            }
            requestBody += 'sms_recipients='+mobiles+'&';
            
            gsr = API_Octopush.sendSMS(false, requestBody,  new map<String, String> {'templateID'=>aSMS.SMS_Template__c, 'receiverType'=>Utils_Octopush.getReceiverType(originID)});
            if(gsr.totalSms == gsr.successSms) {                   
                isSMSSent = 0;
            } else if(gsr.failureSms > 0 && gsr.successSms > 0) {
                isSMSSent = -1;                            
            } else {
                isSMSSent = -99;                   
            }
        }
        return null;        
    }
    
    public PageReference showSendingMsg() {
        if(mandatoryChecking().isEmpty()) {   
             ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, LABEL.INFO_SENDING_SMS)); 
         }
         return null;
    }
    
    public PageReference showSendingResultMsg() {
         if(hasUIValidationError) {
             for(String anError : mandatoryChecking())
                 ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, anError)); 
             return null;
         } else {
             if(isSMSSent == 0) {
                  ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_TOTAL_SMS_SENT+gsr.successSms+'/'+gsr.totalSms));                
                 ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.CONFIRM, Label.CONFIRM_TOTAL_SMS_SENT));
             } else if(isSMSSent == -1) {
                  ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.INFO, Label.INFO_TOTAL_SMS_SENT+gsr.successSms+'/'+gsr.totalSms));      
                 ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.WARNING, Label.WARNING_SOME_SMS_COULD_NOT_BE_SENT));
                 for(API_Octopush.smsResult sr : gsr.smsResult) {
                     if(sr.error_code != null)
                         ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.WARNING, sr.recipient+': '+API_Octopush.getErrorMessageByCode(sr.error_code, 'EN')));
                }
            } else if(isSMSSent == -99){
                  ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, Label.ERROR_SMS_FAILED_TO_BE_SENT));
                 for(API_Octopush.smsResult sr : gsr.smsResult) {
                    if(sr.error_code != null)
                        ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, sr.recipient+': '+API_Octopush.getErrorMessageByCode(sr.error_code, 'EN')));
                }
             } else {
                ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, Label.ERROR_UNKNOWN_ERROR));
             }
        }
        return null;
    }   
    
    public PageReference doCancel() {        
        Map<string,string> URLParameters = ApexPages.currentPage().getParameters();
        PageReference pageRef = new PageReference('/home/home.jsp');
        if(URLParameters.containsKey('retURL')){
            pageRef = new PageReference(URLParameters.get('retURL'));
        }  
        return pageRef;
    }
    
    private list<String> mandatoryChecking() {
        hasUIValidationError = false; 
        list <String> errorList = new list<String>();
        if(String.isEmpty(aSMS.Mobile_Number__c)) {
            errorList.add(Label.ERROR_MISSING_MOBILE_PHONE);
            hasUIValidationError = true;          
        }
        if(String.isEmpty(aSMS.SMS_Message__c)) {
            errorList.add(Label.ERROR_MISSING_MESSAGE);
            hasUIValidationError = true;          
        }
        if(isDefferedSending && aSMS.Sending_Date__c == null) {
            errorList.add(Label.ERROR_SENDING_DATE_MESSAGE);
            hasUIValidationError = true;           
        }
        if(isDefferedSending && aSMS.Sending_Date__c != null && aSMS.Sending_Date__c < DateTime.now()) {
            errorList.add(Label.ERROR_DEFFERED_DATE_NOT_SET_IN_PAST);
            hasUIValidationError = true; 
        }
        return errorList;
    }
    
    public List<SelectOption> getItems() {
        List<SelectOption> options = new List<SelectOption>(); 
        options.add(new SelectOption('Marketing','Marketing SMS')); 
        options.add(new SelectOption('Transactional','Transactional SMS')); return options;         
    }
}