/*
    Testing class for VF_SendSingleSMS_CTR page
*/
@isTest
public class VF_SendSingleSMS_CTRL_TEST {
// Init data for testing
    static {
        Octopush_UtilsTesting.initTestingContext_Contacts();
        Octopush_UtilsTesting.initTestingContext_Leads();
        // Set mock callout class
		Test.setMock(HttpCalloutMock.class, new MockHttpResponseGenerator_TEST());
    }

     public static testMethod void T1_OctopushSettingsNotSet() {
        Test.startTest();
            PageReference pageRef = Page.VF_SendSingleSMS;
            pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.contacts_All[0].Id));
            Test.setCurrentPage(pageRef);        
            VF_SendSingleSMS_CTRL ctrl = new VF_SendSingleSMS_CTRL();
            // Error message because Octopush settings where not set yet
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.WARNING_MISSING_OCTOPUSH_SETTINGS));
        Test.stopTest();
    }
    
    public static testMethod void T2_Contact_SendSMS() {
    	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
    	Octopush_UtilsTesting.initTestingContext_SMSTemplate();
    	
        Test.startTest();
            PageReference pageRef = Page.VF_SendSingleSMS;
            pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.contacts_All[0].Id));
            pageRef.getParameters().put('retURL', '/home/home.jsp');
            Test.setCurrentPage(pageRef);        
            VF_SendSingleSMS_CTRL ctrl = new VF_SendSingleSMS_CTRL();
            ctrl.getItems();
            ctrl.aSMS.Sender_Name__c='test';
			ctrl.aSMS.SMS_Template__c = Octopush_UtilsTesting.templates_All[1].id;
            ctrl.fillTemplateContent();
            ctrl.showSendingMsg();
            ctrl.sendSMS(); // Send 1 SMS with no error
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.CONFIRM_TOTAL_SMS_SENT));
		Test.stopTest();
		system.assertEquals([SELECT id FROM SMS_History__c].size(), 1);
    	system.assertEquals([SELECT Status__c FROM SMS_History__c WHERE Status__c = 'Error'].size(), 0);
    }
    
    public static testMethod void T3_Lead_SendSMS() {
    	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
    	Octopush_UtilsTesting.initTestingContext_SMSTemplate();
    	
        Test.startTest();
            PageReference pageRef = Page.VF_SendSingleSMS;
            pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.leads_All[0].Id));
            Test.setCurrentPage(pageRef);        
            VF_SendSingleSMS_CTRL ctrl = new VF_SendSingleSMS_CTRL();
            ctrl.aSMS.Sender_Name__c='test';
            ctrl.aSMS.SMS_Message__c='test';
            ctrl.isDefferedSending=true;
            ctrl.aSMS.Sending_Date__c = (DateTime.now()+ (1/6.0));
            ctrl.additionalMobile = '+33600000007';
            ctrl.showSendingMsg();
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.CONFIRM_TOTAL_SMS_SENT));
		Test.stopTest();
		system.assertEquals([SELECT id FROM SMS_History__c].size(), 2);
    	system.assertEquals([SELECT Status__c FROM SMS_History__c WHERE Status__c = 'Error'].size(), 0);
    }
    
    public static testMethod void T4_Contact_NoMobile_SendSMS() {
    	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
    	Octopush_UtilsTesting.initTestingContext_SMSTemplate();
    	
        Test.startTest();
            PageReference pageRef = Page.VF_SendSingleSMS;
            pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.contacts_All[4].Id));
            pageRef.getParameters().put('retURL', '/home/home.jsp');
            Test.setCurrentPage(pageRef);        
            VF_SendSingleSMS_CTRL ctrl = new VF_SendSingleSMS_CTRL();
            // warning message because contact has no mobile phone
			system.assert(Octopush_UtilsTesting.messageContainsValue(Label.INFO_PHONE_NOT_FILLED_FOR_CONTACT));
			// Attempt to send with missing mandatory fields
			ctrl.showSendingMsg();
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_MISSING_MESSAGE));
			// Back to previsou page
			ctrl.doCancel();
        Test.stopTest();
    }
    
    public static testMethod void T5_Lead_NoMobile_SendSMS() {
    	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
    	
        Test.startTest();
            PageReference pageRef = Page.VF_SendSingleSMS;
            pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.leads_All[4].Id));
            Test.setCurrentPage(pageRef);        
            VF_SendSingleSMS_CTRL ctrl = new VF_SendSingleSMS_CTRL();
            // warning message because lead has no mobile phone
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.INFO_PHONE_NOT_FILLED_FOR_LEAD));
			// Fill mobile ok but invalid number for additionnal mobile
			ctrl.aSMS.Mobile_Number__c = '+33600000007';
			ctrl.additionalMobile = '+33100000001';
			ctrl.aSMS.SMS_Message__c='test';
			ctrl.showSendingMsg();
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.WARNING_SOME_SMS_COULD_NOT_BE_SENT));
        Test.stopTest();
    }
    
    public static testMethod void T6_MissingMandatoryData() {
    	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
    	
        Test.startTest();
            PageReference pageRef = Page.VF_SendSingleSMS;
            pageRef.getParameters().put('relatedObject', String.valueOf(Octopush_UtilsTesting.leads_All[4].Id));
            Test.setCurrentPage(pageRef);        
            VF_SendSingleSMS_CTRL ctrl = new VF_SendSingleSMS_CTRL();
            ctrl.isDefferedSending=true;
            ctrl.showSendingMsg();
            ctrl.showSendingResultMsg();
            system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_MISSING_MESSAGE));
            ctrl.aSMS.Sending_Date__c = (DateTime.now()- (1/6.0));
            ctrl.aSMS.SMS_Message__c='test';
            ctrl.aSMS.Mobile_Number__c = '+33100000001';
            ctrl.showSendingMsg();
            ctrl.showSendingResultMsg();     
        	system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_DEFFERED_DATE_NOT_SET_IN_PAST));
        Test.stopTest();
        system.assertEquals([SELECT id FROM SMS_History__c].size(), 0);
    }
    
    public static testMethod void T7_NoSMSSent() {
    	Octopush_UtilsTesting.initTestingContext_OctopushSettings();
    	
        Test.startTest();
            PageReference pageRef = Page.VF_SendSingleSMS;
            Test.setCurrentPage(pageRef);        
            VF_SendSingleSMS_CTRL ctrl = new VF_SendSingleSMS_CTRL();
           
			// Fill mobile and add mobile invalid
			ctrl.additionalMobile = '+33100000007';
			ctrl.aSMS.Mobile_Number__c = '+33100000001';
			ctrl.aSMS.SMS_Message__c='test';
			ctrl.showSendingMsg();
            ctrl.sendSMS();
            ctrl.showSendingResultMsg();
            Boolean hasError = false;
			system.assert(Octopush_UtilsTesting.messageContainsValue(Label.ERROR_SMS_FAILED_TO_BE_SENT));
        Test.stopTest();
    }
    
}