public with sharing class VF_WorkflowSetting_CTRL {
    
    public string objectRelatedTo {get; set;}
    public string fieldRelatedTo {get;set;}
    public string senderName{get;set;}
    public string additionalMobile{get;set;}
    public string message{get;set;}
    public string taskComment{get;set;}
    public string smsPurpose{get;set;}
    
    public SMS_History__c aSMS{get; set;}
    
    public VF_WorkflowSetting_CTRL() {
        objectRelatedTo = 'Contact';
        aSMS = new SMS_History__c(SMS_Purpose__c='Marketing');
        Octopush_Settings__c os = Utils_Octopush.getOctopushSettings();
    	if(os != null) {
    		senderName = os.Default_Sender_Name__c;
       		smsPurpose = os.SMS_Purpose__c;
    	}
    }
    
    public pageReference changeObjectRelatedTo() {return null;}
    
    public PageReference fillTemplateContent() {
    	if (Schema.sObjectType.SMS_Template__c.isAccessible() && Schema.sObjectType.SMS_Template__c.fields.Message__c.isAccessible() && Schema.sObjectType.SMS_Template__c.fields.Sender_Name__c.isAccessible()) {
	        list<SMS_Template__c> templateSelected = [SELECT Message__c, Sender_Name__c FROM SMS_Template__c WHERE id =: aSMS.SMS_Template__c limit 1];
	        if(! templateSelected.isEmpty()) {
	            message = templateSelected[0].Message__c;
	            senderName = templateSelected[0].Sender_Name__c;
	        }
	    } else {
    		ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Insufficent priviliges: You must have at least Read access to SMS Template object and message and sender name field. Please contact your administrator for support.'));
    	}
        return null;
    }

    public pageReference generateTask() {
    	if(String.isBlank(message)) {
    		ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, Label.ERROR_MISSING_MESSAGE));
    		return null;
    	}
    	if(message.length() > 459) {
    		ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.ERROR, 'Message can not exceed 459 characters'));
    		return null;
    	}
        taskComment = 'OBJ='+objectRelatedTo+'|\n';
        taskComment += 'SN='+senderName+'|\n';
        String template = aSMS.SMS_Template__c == null ? '' : aSMS.SMS_Template__c;
        taskComment += 'TMP='+template+'|\n';
        taskComment += 'PUR='+smsPurpose+'|\n';
        taskComment += 'MFI='+fieldRelatedTo+'|\n';
        taskComment += 'ADM='+additionalMobile+'|\n';
        taskComment += 'MSG='+message+'|\n';
        
        ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.CONFIRM, Label.CONFIRM_TASK_GENERATED_1));
        ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.CONFIRM, Label.CONFIRM_TASK_GENERATED_2));
        ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.CONFIRM, Label.CONFIRM_TASK_GENERATED_3));
        ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.CONFIRM, Label.CONFIRM_TASK_GENERATED_4));
        ApexPages.addMessage(new ApexPages.Message(ApexPages.severity.CONFIRM, Label.CONFIRM_TASK_GENERATED_5));
        return null;
    }
    public pageReference cancel() {
		Map<string,string> URLParameters = ApexPages.currentPage().getParameters();
        PageReference pageRef = new PageReference('/home/home.jsp');
        if(URLParameters.containsKey('retURL')){
            pageRef = new PageReference(URLParameters.get('retURL'));
        }  
        return pageRef;
	}
    
    public List<SelectOption> getObjectOptions() {
        List<SelectOption> objectOptions = new List<SelectOption>();         
        objectOptions.add(new SelectOption('Contact', 'Contact'));
        objectOptions.add(new SelectOption('Lead', 'Lead'));
        return objectOptions;
    }    
    
    public List<SelectOption> getFieldsOptions() {
        Map<String, Schema.SObjectField> mapFields = Schema.getGlobalDescribe().get(objectRelatedTo).getDescribe().fields.getMap();
        List<String> sortedKeyset = new List<String>();
        for (String fieldName: mapFields.keyset()){           
            sortedKeyset.add(fieldName);
        }
        sortedKeyset.sort();
        List<SelectOption> fieldOptions = new List<SelectOption>();
         for (String fieldName: sortedKeyset){
            fieldOptions.add(new SelectOption(fieldName,mapFields.get(fieldName).getDescribe().getLabel()));
        }
        return fieldOptions;
    }    
    
    public List<SelectOption> getItems() {
        List<SelectOption> options = new List<SelectOption>(); 
        options.add(new SelectOption('Marketing','Marketing SMS')); 
        options.add(new SelectOption('Transactional','Transactional SMS')); return options;         
    }
}